#include <pic.h>				// pic specific identifiers
#define _XTAL_FREQ  4000000		// Xtal speed
__CONFIG(0x3D18);				// Config bits

#define LCD_PORT 	PORTA		// constants
#define LCD_TRIS	TRISA		//
#define LCD_RS 		RA4
#define LCD_RW		RA6
#define LCD_E		RA7
#define RS232_PORT	PORTB		// RS232 port configuration
#define RS232_TRIS	TRISB
#define RX			RB7
#define TX			RB6
#define BAUD		84			// 1/9600 = 104uS But! with loop and shift @ 20uS
#define START_BIT	BAUD/2		// Should be 52uS really, but close enough

		// Required prototypes.. each function needs to be declared
		// if called BEFORE definition.

void LCD_init(void), LCD_cmd(unsigned char ch);
void LCD_goto(char line, char column), LCD_clr(void);
void LCD_cur(unsigned char ch), pulse_E(void);
void LCD_char(unsigned char ch), LCD_charD(unsigned char ch);
void Serinit(void), SerStringC(const char* str);
void Serout( unsigned char ch);
unsigned char Serin(void);

void main(void)						// program entry
	{
	int index = 0;
	ADCON1 = 0x6;					// Analogue off
	LCD_TRIS = 0b00000000;			// Led port as outputs
	__delay_ms(150);				// let LCD stabilise
	LCD_init();						// Initalise screen
	LCD_goto(1,0);					// line 1.
	HSerinit();
	while(1)						// endless Loop
		{
		ch = HSerin();				// wait for a character

		if(index == 16)				// new line
			LCD_goto(2,0);
		if(index == 32)				// goto line 1
			{
			LCD_clr();
			index = 0;				// re-set
			}
		LCD_char(ch);				// display the received byte
		HSerout(ch);					// Echo back
		index++;					// move on
		}
	}

void Serinit()
	{
	RS232_TRIS = 0b10000000;			// RS232  port
	TX = 1;
	}

unsigned char Serin()					// wait and get a byte on the serial
	{
	char index, ch=0;
	while(1)
		{
		while(RX);						// wait for start bit
		__delay_us(START_BIT);			// Half bit delay
		if(!RX)							// still start?
			{
			__delay_us(BAUD);				// bit delay
			for(index = 0;index<7;index++)
				{
				if(RX)						// bit or no bit
					ch+=0x80;				// add MSB
				__delay_us(BAUD);			// bit delay
				ch>>=1;						// shift along
				}							// until done
			if(RX) ch+=0x80;				// last bit
			return ch;						// return character
			}								// not bothered about the stop bit
		}
	}

void Serout( unsigned char ch)
	{
	char index;
	TX = 0;									// send a start bit
	__delay_us(BAUD);
	for(index = 0; index<8 ; index++)		// send the rest
		{
		if(ch & 0x1)						// test bit
			TX = 1;							// send high
		else
			TX = 0;							// send low
		__delay_us(BAUD);					// bit delay
		ch >>= 1;
		}
	TX = 1;									// send a stop bit
	__delay_us(BAUD);
	}

void LCD_init()
	{
	LCD_cmd(0x20);					// 4 bit
	LCD_cmd(0x28);					// display shift
	LCD_cmd(0x6);					// character mode
	LCD_cmd(0xc);					// display on / off and cursor
	LCD_clr();						// clear display
	}


void LCD_goto(char line, char column)		// combines line and lineW
	{
	unsigned char data = 0x80;				// default to 1
	if(line == 2)data = 0xc0;				// change to 2
	data += column;							// add in  column
	LCD_cmd(data);
	}

void LCD_clr()
	{
	LCD_cmd(1);								// Clr screen
	}

void LCD_cur(char on)
	{
	unsigned char cur = 0xc;				// cursor off
	if(on) cur = 0xd;						// cursor on
	LCD_cmd(cur);
	}

void LCD_cmd(unsigned char ch)
	{
	LCD_PORT = ch >> 4 & 0xf;			// write high nibble
	LCD_RS = 0;
	pulse_E();
	LCD_PORT = ch & 0xf;				// write low nibble
	LCD_RS = 0;
	pulse_E();
	__delay_ms(5);
	}

void LCD_charD(unsigned char ch)
	{
	ch+=0x30;
	LCD_char(ch);						// convert to ascii
	}

void LCD_char(unsigned char ch)
	{
	LCD_PORT = ch >> 4 & 0xf;			// High nibble
	LCD_RS = 1;
	pulse_E();
	LCD_PORT = ch & 0xf;				// low nibble
	LCD_RS = 1;
	pulse_E();
	__delay_ms(5);
	}

void pulse_E()
	{
	LCD_E = 1;
	__delay_us(1);
	LCD_E = 0;
	}