#include <pic.h>				// pic specific identifiers
#define _XTAL_FREQ  4000000		// Xtal speed
__CONFIG(0x3D18);				// Config bits

#define LED_PORT	PORTB		// LED port
#define LED_TRIS	TRISB
#define RS232_PORT	PORTA		// RS232 port configuration
#define RS232_TRIS	TRISA
#define RX			RA7
#define TX			RA6
#define BAUD		84			// 1/9600 = 104uS But! with loop and shift @ 20uS
#define START_BIT	BAUD/2		// Should be 52uS really, but close enough

		// Required prototypes.. each function needs to be declared
		// if called BEFORE definition.

void Serinit(void), SerStringC(const char* str);
void Serout( unsigned char ch);
unsigned char Serin(void);

void main(void)						// program entry
	{
	int index = 0;
	char ch;						// new buffer variable
	CMCON = 0x7;					// Comparitors off
	LED_TRIS = 0b00000000;			// Led port as outputs
	__delay_ms(150);				// let LCD stabilise
	Serinit();
	LED_PORT = 0;
	while(1)						// endless Loop
		{
		ch = Serin();				// wait for a character
		LED_PORT = ch;				// display on portb
		}
	}

void SerStringC(const char* str)	// call serout multiple times
	{
	while(*str != 0)				// until end of string
		Serout(*str++);				// If you use a RAM pointer
	}								// change "const" to "unsigned"

void Serinit()
	{
	RS232_TRIS = 0b10000000;			// RS232  port
	TX = 1;								// initial tx value
	}

unsigned char Serin()					// wait and get a byte on the serial
	{
	char index, ch=0;
	while(1)
		{
		while(RX);						// wait for start bit
		__delay_us(START_BIT);			// Half bit delay
		if(!RX)							// still start?
			{
			__delay_us(BAUD);				// bit delay
			for(index = 0;index<7;index++)
				{
				if(RX)						// bit or no bit
					ch+=0x80;				// add MSB
				__delay_us(BAUD);			// bit delay
				ch>>=1;						// shift along
				}							// until done
			if(RX) ch+=0x80;				// last bit
			return ch;						// return character
			}								// not bothered about the stop bit
		}
	return 0;
	}

void Serout( unsigned char ch)
	{
	char index;
	TX = 0;									// send a start bit
	__delay_us(BAUD);
	for(index = 0; index<8 ; index++)		// send the rest
		{
		if(ch & 0x1)						// test bit
			TX = 1;							// send high
		else
			TX = 0;							// send low
		__delay_us(BAUD);					// bit delay
		ch >>= 1;
		}
	TX = 1;									// send a stop bit
	__delay_us(BAUD);
	}

